import json
import os
import time
import traceback
from datetime import datetime
from enum import Enum
from typing import Optional

import requests

from korbit.constant import (
    KORBIT_CODE_ANALYSIS_CHECK,
    KORBIT_CODE_ANALYSIS_PULL_REQUEST_CHECK,
    KORBIT_CODE_ANALYSIS_SERVICE,
    KORBIT_LOCAL_FOLDER,
    KORBIT_SCAN_REPORT_URL,
)
from korbit.interface import (
    INTERFACE_SCAN_REPORT_ISSUES_COUNT_MSG,
    INTERFACE_SCAN_WAITING_REPORT_MSG,
    INTERFACE_SCAN_WAITING_START_MSG,
    INTERFACE_SLEEPING_REFRESH,
    console_print_message,
    construct_file_tree,
    create_console,
    create_progress_bar,
    generate_category_table,
)
from korbit.login import authenticate_request
from korbit.models.issue import IssueFilterThresholds
from korbit.models.report import Report, ReportCategory
from korbit.telemetry import send_telemetry


class ProgressStatus(Enum):
    SUCCESS = "SUCCESS"
    FAILURE = "FAILURE"
    PROGRESS = "PROGRESS"


def display_scan_status(scan_id: int, headless=False):
    """
    Display the progress of the scan status.
    If it's headless mode, the messages will be prompt only once.
    """
    console = create_console(headless)
    previous_progress = -1
    while True:
        response = authenticate_request(requests.get, url=f"{KORBIT_CODE_ANALYSIS_SERVICE}/{scan_id}/progress")
        try:
            data = response.json()
            status = data.get("status")
            if not status:
                console_print_message(console, INTERFACE_SCAN_WAITING_START_MSG, only_once=headless)
                time.sleep(INTERFACE_SLEEPING_REFRESH)
                continue
            if status == ProgressStatus.SUCCESS.value:
                console_print_message(console, INTERFACE_SCAN_WAITING_REPORT_MSG, only_once=headless)
                break

            progress = data.get("progress", 0.0)

            title = data.get("title", "File(s) status")

            file_tree_data = data.get("files", [])
            tree = construct_file_tree(title, file_tree_data)

            progress_bar = create_progress_bar(console, f"Analyzing files ({len(file_tree_data)})...", progress)
            if headless and previous_progress == progress:
                continue
            console.clear()
            console.print(tree)
            console.print(progress_bar)

        except Exception as e:
            send_telemetry([""], f"display_scan_status: {traceback.format_exc()}", error=True)
            console.print(f"Error processing response: {e}")

        time.sleep(INTERFACE_SLEEPING_REFRESH)


def download_report(scan_id: int) -> dict:
    response = authenticate_request(
        requests.get, url=f"{KORBIT_SCAN_REPORT_URL}/{scan_id}/issues?format=json&output_concept_embedding=false"
    )

    now = datetime.now()
    datetime_format = "%Y-%m-%dT%H-%M-%S"
    report_path = os.path.join(KORBIT_LOCAL_FOLDER, f"{scan_id}_{now.strftime(datetime_format)}_report")
    os.makedirs(KORBIT_LOCAL_FOLDER, exist_ok=True)
    html_report_path = f"{report_path}.html"
    json_report_path = f"{report_path}.json"

    with open(os.path.abspath(json_report_path), "w+") as json_file:
        issues = response.content.decode()
        json_file.write(issues)
        all_issues = json.loads(issues)

    final_issues = {"issues": all_issues, "scan_id": scan_id, "report_path": html_report_path}

    return final_issues


def filter_issues_by_threshold(report: Report, thresholds: IssueFilterThresholds) -> Report:
    if not thresholds:
        return report
    filtered_categories = []
    for category, issues in report.categories_iterator():
        filtered_categories.append(ReportCategory(category, thresholds.apply(issues)))

    report.categories = filtered_categories
    return report


def display_report(report: Report, headless=False) -> tuple[int, int]:
    console = create_console(headless)
    total_issues, selected_issues, ignored_issues = report.get_issues_stats()
    for i, category_issues in enumerate(report.categories):
        if not category_issues.get_total_selected_issues():
            continue

        generate_category_table(console, category_issues.category, category_issues.issues)

        if i != len(report.categories) - 1:
            console.print("\n")
            console.print("\n")

    console.print(
        INTERFACE_SCAN_REPORT_ISSUES_COUNT_MSG.format(
            total_issues=total_issues, selected_issues=selected_issues, ignored_issues=ignored_issues
        )
    )

    with open(report.report_path, "a+", encoding="utf-8") as report_file:
        report_file.write(console.export_html())

    return total_issues, selected_issues


def upload_file(zip_file_path: str, is_pr_scan: bool = False) -> Optional[int]:
    check_url = KORBIT_CODE_ANALYSIS_CHECK
    if is_pr_scan:
        check_url = KORBIT_CODE_ANALYSIS_PULL_REQUEST_CHECK
    with open(zip_file_path, "rb") as file:
        response = authenticate_request(requests.post, url=check_url, files={"repository": file})
        if response.status_code == 200:
            return response.json()
        else:
            return None



import urllib.parse
from pathlib import Path
from typing import Optional

import git


class BranchNotFound(Exception):
    def __init__(self, branch_name):
        super().__init__(f"Branch not found: {branch_name}")
        self.branch_name = branch_name


def branch_exists(repo: git.Repo, branch_name):
    return any(branch.name == branch_name for branch in repo.branches + repo.refs)


def get_repository(repository_path: Path) -> git.Repo:
    return git.Repo(repository_path)


def get_commits_head_and_compare(repo: git.Repo, compare_branch: str) -> tuple[str, str]:
    if not branch_exists(repo, compare_branch):
        raise BranchNotFound(compare_branch)
    compare_commit_head_hash = repo.git.rev_parse(compare_branch)
    current_commit_head_hash = repo.head.commit.hexsha

    return compare_commit_head_hash, current_commit_head_hash


def get_diff_paths(repo_path: Path, compare_branch: str) -> list[str]:
    repo = get_repository(repo_path)
    commit_head, commit_compare = get_commits_head_and_compare(repo, compare_branch)
    diff = repo.git.diff(commit_head, commit_compare, name_only=True)
    return diff.split("\n") if diff else []


def get_diff_per_files(repo_path: Path, compare_branch: str) -> dict[str, str]:
    repo = get_repository(repo_path)
    commit_head, commit_compare = get_commits_head_and_compare(repo, compare_branch)
    diff_paths = get_diff_paths(repo_path, compare_branch)
    diff_dict = {path: repo.git.diff(commit_head, commit_compare, path) for path in diff_paths}
    return diff_dict


def get_head_branch_info(repo):
    head_commit = repo.head.commit
    return {
        "ref": head_commit.name_rev,
        "commit_sha": head_commit.hexsha,
        "commit_author": head_commit.author.name,
        "commit_date": head_commit.committed_datetime.isoformat(),
    }


def get_base_branch_info(repo: git.Repo, base_branch: str):
    target_commit = repo.commit(base_branch)
    return {
        "ref": base_branch,
        "commit_sha": target_commit.hexsha,
        "commit_author": target_commit.author.name,
        "commit_date": target_commit.committed_datetime.isoformat(),
    }


def extract_repository_owner_and_name(repository_url: str) -> tuple[str, str]:
    """Handle 'git@' and 'https' repository url"""
    parsed_url = urllib.parse.urlparse(repository_url)
    path_parts = parsed_url.path.split("/")
    if len(path_parts) < 2:
        return "", ""
    if not path_parts[0] and len(path_parts) >= 3:
        path_parts = path_parts[1:]
    repo_owner = path_parts[0]
    if repo_owner.startswith("git@"):
        repo_owner = repo_owner.split(":")[-1]
    repo_name = path_parts[1]
    if repo_name.endswith(".git"):
        repo_name = repo_name[:-4]
    return repo_owner, repo_name


def get_repository_metadata(repository_path: Path, base_branch: Optional[str] = None):
    repo = get_repository(repository_path)

    head_branch_info = get_head_branch_info(repo)

    repository_info = {
        "head_branch": head_branch_info,
        "default_branch": repo.heads[0].name if repo.heads else "",
    }
    if base_branch:
        base_branch_info = get_base_branch_info(repo, base_branch)
        repository_info["base_branch"] = base_branch_info
    repository_url = repo.remotes.origin.url if repo.remotes else None
    if repository_url:
        repository_owner, repository_name = extract_repository_owner_and_name(repository_url)
        repository_info["repository_owner"] = repository_owner
        repository_info["repository_name"] = repository_name
        repository_info["repository_url"] = repository_url
    return repository_info


def get_repository_metadata_with_diff(repository_path: Path, base_branch: str):
    diff_dict = get_diff_per_files(repository_path, base_branch)
    repository_info = get_repository_metadata(repository_path, base_branch)
    repository_info["pull_request"] = diff_dict
    return repository_info



from dataclasses import dataclass, field


@dataclass
class IssueFile:
    """
    This class represents a file where an issue was found.

    Attributes:
    confidence: Confidence level of the issue found in the file.
    priority: Priority level of the issue found in the file.
    path: Path to the file.
    line_start: Line where the issue starts.
    line_end: Line where the issue ends.
    """

    confidence: int
    priority: int
    path: str
    line_start: int
    line_end: int

    @classmethod
    def from_json(cls, json_data):
        return cls(
            confidence=json_data["confidence"],
            priority=json_data["priority"],
            path=json_data["path"],
            line_start=json_data["line_start"],
            line_end=json_data["line_end"],
        )

    def get_line_number(self):
        if self.line_start == self.line_end:
            return self.line_start
        return f"{self.line_start}:{self.line_end}"


@dataclass
class Issue:
    """
    This class represents an issue found in one or more files.

    Attributes:
    description: Description of the issue.
    issue_files: List of files where the issue was found.
    """

    description: str
    code_suggestion: str
    issue_files: list[IssueFile]
    ignored_issue_files: list[IssueFile] = field(default_factory=list)

    @classmethod
    def from_json(cls, json_data):
        return cls(
            description=json_data["description"],
            issue_files=[IssueFile.from_json(file) for file in json_data["files"]],
            code_suggestion=json_data.get("code_suggestion", ""),
        )

    @property
    def language(self) -> str:
        file_path = self.issue_files[0].path
        return file_path.split(".")[-1] if "." in file_path else ""


@dataclass
class IssueFilterThresholds:
    """
    This class represents the thresholds for filtering issues.

    Attributes:
    confidence: Confidence threshold (scale 0-10).
    priority: Priority threshold (scale 0-10).
    """

    confidence: int
    priority: int

    def apply(self, issues: list[Issue]) -> list[Issue]:
        filtered_issues = []
        for issue in issues:
            selected_issue_files = []
            ignored_issue_files = []
            for issue_file in issue.issue_files:
                if issue_file.confidence >= self.confidence and issue_file.priority >= self.priority:
                    selected_issue_files.append(issue_file)
                else:
                    ignored_issue_files.append(issue_file)

            filtered_issues.append(
                Issue(
                    description=issue.description,
                    issue_files=selected_issue_files,
                    ignored_issue_files=ignored_issue_files,
                    code_suggestion=issue.code_suggestion,
                )
            )
        return filtered_issues



import os

from rich.console import Console
from rich.markdown import Markdown
from rich.progress import Progress
from rich.table import Table
from rich.tree import Tree

from korbit.constant import KORBIT_LOCAL_OUTPUT_LOG_FILE
from korbit.models.issue import Issue

# SCAN
INTERFACE_SCAN_COMMAND_HELP = """
Request a scan on a given path of a local file or folder.\n
Example:\n\n
`korbit scan /path/to/folder`\n
`korbit scan path/to/folder`\n
`korbit scan .`\n
"""
INTERFACE_SCAN_PATH_HELP = "Put the absolute or relative path of a local folder or file to scan."
INTERFACE_SCAN_THRESHOLD_PRIORITY_HELP = (
    "When specified, this threshold will filter out issues only above this priority score (on a scale of 0-10)."
)
INTERFACE_SCAN_THRESHOLD_CONFIDENCE_HELP = (
    "When specified, this threshold will filter out issues only above this confidence score (on a scale of 0-10)."
)
INTERFACE_SCAN_HEADLESS_OUTPUT_REPORT_HELP = (
    "On headless mode, the report will be printed only at the end of the terminal output, not in real-time."
)
INTERFACE_SCAN_NO_FILE_FOUND_MSG = (
    "For the given path, no file was found to be scanned. In that case the scan is skipped."
)
INTERFACE_SCAN_EXCLUDE_PATHS_HELP = (
    "Exclude paths from being scanned by adding them to the `--exclude-paths` command option comma separated."
)
INTERFACE_SCAN_EXIT_CODE_HELP = """Run scan in headless mode.
Output will be generated in the .korbit folder in the current working directory.\n
If anything goes wrong, the following exit codes could be returned:\n
- 90: something went wrong (unknown problem, contact support)\n
- 91: Check found issues\n
- 92: Can not start the folder/file analysis\n
- 93: authentication failed\n
- 94: file is too large\n
- 95: korbit license limit reached\n
- 96: korbit mentor version is outdated
"""
INTERFACE_SCAN_EXCLUDE_PATHS_HELP = """You can exclude paths from being scanned by adding them to the exclude_paths argument comma separated. Example:\n
--exclude-paths="**/.git/*,**/node_modules/*"""
INTERFACE_SCAN_REPORT_ISSUES_COUNT_MSG = (
    "Final report contains {selected_issues} issues We ignored {ignored_issues}/{total_issues}."
)
INTERFACE_SCAN_WAITING_START_MSG = "Analysis requested is in the queue, it will start shortly..."
INTERFACE_SCAN_WAITING_REPORT_MSG = "Analysis completed successfully! Report generation in progress..."
INTERFACE_SOMETHING_WENT_WRONG_MSG = (
    "Sorry about that, something went wrong. Please contact support@korbit.ai for help."
)
INTERFACE_CHECK_FAILED_MSG = (
    "Sorry, we are not able to start the analysis of the file(s). Please contact support@korbit.ai for more help."
)
INTERFACE_SCAN_UPLOAD_FAILED_DUE_TO_SIZE_MSG = """You can't upload more than 100MB of files to Korbit mentor.
Please make sure you select only the relevant files to be analyzed, and try again. 
You can select specific folders or files to ignore by adding them to the exclude_paths argument."""
INTERFACE_SCAN_FINAL_REPORT_FOR_HEADLESS_MSG = "We found some issue in the code, see final report {path}"
INTERFACE_SCAN_FINAL_REPORT_PATH_MSG = "You can access the report at {path}"
INTERFACE_SCAN_PREPARING_FOLDER_SCAN_MSG = "Preparing to scan: {paths}"
INTERFACE_SCAN_REQUESTING_A_SCAN_MSG = "Requesting a scan for: {path}"
INTERFACE_SCAN_LICENSE_LIMIT_REACHED_MSG = "You have reached the limits of your current license and this functionality is no longer available. To add a Korbit paid license please visit https://www.korbit.ai/pricing."
INTERFACE_SCAN_OUTPUT_TABLE_NO_SUGGESTION_MSG = "_No code suggestion_"
# SCAN PR
INTERFACE_SCAN_PR_COMMAND_HELP = """
Request a scan on a pull request. You must be running the command on a path that is git repository.\n
The given compare_branch should exist in the repository. If you run this locally, make sure you have pulled the branch, and it's up-to-date.\n
Example:\n\n
`korbit scan-pr /path/to/repository master`\n
`korbit scan-pr path/to/repository master`\n
`korbit scan-pr # Default use current directory (`.`), as the repository and `origin/master` as the base branch"""
INTERFACE_SCAN_PR_NOT_GIT_REPOSITORY_MSG = "The repository path isn't a valid git repository."
INTERFACE_SCAN_PR_EXIT_CODE_HELP = f"""{INTERFACE_SCAN_EXIT_CODE_HELP}\n\n
- 190: Unknown git error\n
- 191: Not a git repository\n
- 192: Branch not found"""

INTERFACE_SCAN_PR_GIT_COMMAND_ERROR_MSG = "Git command failed with exit code {status_code}: {error_message}"
INTERFACE_SCAN_PR_GIT_BRANCH_NOT_FOUND_MSG = "Branch {branch} not found in the repository."
INTERFACE_SCAN_PR_GIT_IS_NOT_INSTALLED_MSG = (
    "You must have git installed to run this command. Follow this link: https://git-scm.com/downloads"
)
# AUTH
INTERFACE_AUTH_COMMAND_HELP = """Store user credentials for future usage of `korbit scan --help` command. 
If you don't provide any arguments to the command, you will be requested to input the values.\n
You can set the following environment variables to replace this command:\n
```sh\n
export KORBIT_SECRET_ID=xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx\n
export KORBIT_SECRET_KEY=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n
```\n
"""
INTERFACE_AUTH_UNAUTHORIZED_CREDENTIALS_MSG = "Invalid user credentials, please login again with `korbit login`!"
INTERFACE_AUTH_MISSING_CREDENTIALS_MSG = "No user credentials found, please login first with `korbit login`!"
INTERFACE_AUTH_INPUT_SECRET_ID = "Please enter your secret ID: "
INTERFACE_AUTH_INPUT_SECRET_ID_HELP = "The secret_id value generated on Korbit app for you user."
INTERFACE_AUTH_INPUT_SECRET_KEY = "Please enter your secret key (hidden): "
INTERFACE_AUTH_INPUT_SECRET_KEY_HELP = "The secret_key generated on Korbit app for you user."

# GENERAL
INTERFACE_UPDATE_REQUIRED_MSG = """
A new version of Korbit CLI is available, follow the documentation here: https://docs.korbit.ai/#/cli/update.md

If you install the CLI with pip you can run the following command:

```sh
pip install --upgrade korbit-mentor
```
"""

INTERFACE_SLEEPING_REFRESH = 2

tick_rotation = False


def create_console(headless: bool):
    if headless:
        return Console(file=open(KORBIT_LOCAL_OUTPUT_LOG_FILE, "a+"), record=True)

    return Console(record=True)


def create_progress_bar(console: Console, message: str, curr_progress: int = 0):
    with Progress(console=console, auto_refresh=True) as progress_bar:
        task = progress_bar.add_task(message, total=100, spinner="⌛️")
        progress_bar.update(task, completed=curr_progress)
        return progress_bar


def console_print_message(console: Console, message: str, only_once: bool = False, **kwargs):
    if only_once and message in console.export_text(clear=False):
        return
    console.clear()
    console.print(message, **kwargs)


def construct_file_tree(title: str, file_list: list) -> Tree:
    file_tree = build_file_tree(file_list)
    top_tree_node = Tree(title)
    add_nodes_to_tree(top_tree_node, file_tree)
    return top_tree_node


def build_file_tree(file_list: list) -> dict:
    file_tree = {}
    for file_info in file_list:
        file_name = file_info["name"]
        file_status = file_info.get("status", "Unknown")

        directory, filename = os.path.split(file_name)

        current_node = file_tree
        for folder in directory.split(os.sep):
            if folder not in current_node:
                current_node[folder] = {}
            current_node = current_node[folder]

        current_node[filename] = file_status

    return file_tree


def create_tree_node(name, status):
    global tick_rotation
    if status == "PROGRESS":
        name_with_status = name + (" ⌛️" if tick_rotation else " ⏳")
    elif status == "FAILURE":
        name_with_status = name + " ❌"
    elif status == "SUCCESS":
        name_with_status = name + " ✅"
    else:
        name_with_status = name + " 👀"
    node = Tree(name_with_status)
    tick_rotation = not tick_rotation
    return node


def add_nodes_to_tree(tree_node, current_node):
    for name, value in current_node.items():
        if isinstance(value, dict):
            if name:
                node = tree_node.add(name)
            else:
                node = tree_node
            add_nodes_to_tree(node, value)
        else:
            tree_node.add(create_tree_node(name, value))


def compute_issue_description(issue: Issue):
    text = issue.description
    if issue.code_suggestion:
        text += f"\n\nCode suggestion:\n\n```{issue.language}\n{issue.code_suggestion}\n```"
    else:
        text += "\n\n" + INTERFACE_SCAN_OUTPUT_TABLE_NO_SUGGESTION_MSG
    return Markdown(text)


def generate_category_table(console: Console, category: str, issues: list[Issue]):
    table = Table(title=f"Category: {category}")
    table.add_column("Error Description", overflow="fold")
    table.add_column("File Path", overflow="fold")
    table.add_column("Confidence")
    table.add_column("Priority")

    for issue in issues:
        for j, issue_file in enumerate(reversed(sorted(issue.issue_files, key=lambda i_file: i_file.priority))):
            description = compute_issue_description(issue) if j == 0 else ""
            is_last_element = j == len(issue.issue_files) - 1
            table.add_row(
                description,
                f"{issue_file.path}:{issue_file.get_line_number()}",
                str(issue_file.confidence),
                str(issue_file.priority),
                end_section=is_last_element,
            )
    console.print(table)


def print_markdown_message(message: str):
    console = create_console(headless=False)
    console.print(Markdown(message))



import fnmatch
import json
import os
import shutil
import tempfile
import traceback
import zipfile
from pathlib import Path
from typing import IO
from uuid import uuid4

from korbit.constant import (
    KORBIT_LOCAL_OUTPUT_LOG_FILE,
    KORBIT_LOCAL_REPOSITORY_METADATA_FILENAME,
    KORBIT_LOCAL_REPOSITORY_METADATA_PATH,
)
from korbit.telemetry import send_telemetry

TEMP_FOLDER = f"{uuid4()}/"


class ZipfileEmptyError(Exception):
    pass


def compute_folder_name(_path: Path) -> str:
    if str(_path.parent).startswith(".."):
        return "parent_dir"
    elif str(_path.parent).startswith("."):
        return "current_dir"
    return f"{TEMP_FOLDER}/{_path.name}"


def compute_folder_name_for_file(_path: Path) -> str:
    folder_name = compute_folder_name(_path)
    if folder_name == _path.name:
        return str(_path.parent)
    return folder_name


def find_common_prefix(paths: list[str]) -> str:
    """Find the common prefix among the paths. If there is no common prefix, a temporary folder name is returned."""
    # If we have a single path we can still try to get the parent or folder
    common_prefix = os.path.commonprefix(paths)
    if len(paths) == 1:
        path_obj = Path(paths[0])
        if path_obj.is_file():
            common_prefix = str(path_obj.parent)
        else:
            common_prefix = path_obj.name
    # If there is no common prefix, create temporary name
    if not common_prefix:
        common_prefix = TEMP_FOLDER
    return common_prefix


def create_destination_path(temp_folder: Path, path_obj: Path, common_prefix: str) -> Path:
    """Creates a destination path based on the given temporary folder, path object, and common prefix"""
    destination_path = temp_folder / TEMP_FOLDER
    if path_obj.is_absolute():
        # Get the relative path of the file within the temporary folder
        # common_prefix will be assign to TEMP_FOLDER if there is no common prefix so we use the "/"
        rel_path = path_obj.relative_to("/" if common_prefix == TEMP_FOLDER else common_prefix)
        if str(rel_path.parent) != ".":
            destination_path /= rel_path.parent
    else:
        destination_path /= path_obj.parent
    # Create the directories in the temporary folder if they don't exist
    destination_path.mkdir(parents=True, exist_ok=True)
    return destination_path


def get_zip_top_folder_and_name(common_prefix: str) -> tuple[str, str]:
    highest_dir = os.path.basename(common_prefix.rstrip(os.sep))
    highest_dir = compute_folder_name(Path(highest_dir))
    return highest_dir, highest_dir + ".zip"


def chmod_recursive(path_folder: Path):
    for root, dirs, files in os.walk(path_folder):
        try:
            for d in dirs:
                os.chmod(os.path.join(root, d), 0o777)
            for f in files:
                os.chmod(os.path.join(root, f), 0o777)
        except PermissionError:
            send_telemetry(
                [""], "Something went wrong while changing permission: " + traceback.format_exc(), error=True
            )


def rmtree_temporary_safely(path_folder: Path):
    "Change permission to 777 recursively to the folder and delete it."
    chmod_recursive(path_folder)
    shutil.rmtree(path_folder)


def create_temp_folder(zip_file_name: str) -> Path:
    temp_folder = Path(tempfile.gettempdir()) / zip_file_name
    try:
        temp_folder.mkdir(parents=True)
    except FileExistsError:
        rmtree_temporary_safely(temp_folder)
    return temp_folder


def copy_safely(src: str, dest: str):
    try:
        if len(dest) >= 260 or len(src) >= 260:
            return
        shutil.copy(src, dest, follow_symlinks=False)
    except PermissionError:
        pass
    except FileNotFoundError:
        pass
    except OSError:
        pass


def copytree_safely(src: str, destination: str, ignore_paths):
    try:
        shutil.copytree(
            src,
            destination,
            dirs_exist_ok=True,
            ignore_dangling_symlinks=True,
            ignore=shutil.ignore_patterns(*ignore_paths),
            copy_function=copy_safely,
        )
    except (shutil.Error, OSError):
        send_telemetry([""], "Something went wrong while copying tree: " + traceback.format_exc(), error=False)


def copy_files_to_temp_folder_with_ignore(
    paths: list[str], temp_folder: Path, common_prefix: str, ignore_paths: list[str]
) -> None:
    for path in paths:
        path_obj = Path(path)
        if any(fnmatch.fnmatch(path, pattern) for pattern in ignore_paths):
            continue
        if path_obj.is_file():
            destination_path = create_destination_path(temp_folder, path_obj, common_prefix)
            copy_safely(path, str(destination_path / path_obj.name))
        elif path_obj.is_dir():
            destination_path = compute_folder_name(path_obj)
            copytree_safely(path, str(temp_folder / destination_path), ignore_paths)


def create_zip_file(temp_folder: Path, zip_file_name: str, exclude_paths: list[str]) -> str:
    with zipfile.ZipFile(zip_file_name, "w") as zipf:
        for file in temp_folder.rglob("*"):
            arcname = file.relative_to(temp_folder)
            # Apply exclude rules to zip creation
            if any(fnmatch.fnmatch(str(arcname), path_rule.strip()) for path_rule in exclude_paths):
                continue
            zipf.write(file, arcname=arcname)
        if len(zipf.filelist) == 0:
            raise ZipfileEmptyError("The zip file is empty.")
    return zip_file_name


def get_korbit_ignore(paths: list[Path], exclude_paths: list[str]) -> list[str]:
    final_ignore_paths = exclude_paths
    for path in paths:
        path_obj = Path(path)
        if path_obj.is_dir():
            korbit_ignore_path = path_obj / ".korbitignore"
            if korbit_ignore_path.exists():
                with korbit_ignore_path.open("r") as file:
                    for line in file:
                        final_ignore_paths.append(line.strip())
    return final_ignore_paths


def get_stored_repository_metadata() -> dict:
    if not os.path.exists(KORBIT_LOCAL_REPOSITORY_METADATA_PATH):
        return {}
    with open(KORBIT_LOCAL_REPOSITORY_METADATA_PATH, "r+") as file:
        try:
            metadata = json.load(file)
        except json.JSONDecodeError:
            send_telemetry(
                [""],
                f"Failed to load repository_metadata (scan continue..): {traceback.format_exc()}\n{file.read()}",
                error=True,
            )
            metadata = {}
    return metadata


def repository_metadata_update(data: dict):
    metadata = get_stored_repository_metadata()

    metadata.update(data)

    with open(KORBIT_LOCAL_REPOSITORY_METADATA_PATH, "w+") as file:
        json.dump(metadata, file)


def add_repository_metadata(temp_folder: Path):
    if os.path.exists(KORBIT_LOCAL_REPOSITORY_METADATA_PATH):
        try:
            shutil.copy(
                KORBIT_LOCAL_REPOSITORY_METADATA_PATH,
                temp_folder / TEMP_FOLDER / KORBIT_LOCAL_REPOSITORY_METADATA_FILENAME,
            )
        except Exception:
            send_telemetry(
                [""], f"Failed to add repository_metadata (scan continue..): {traceback.format_exc()}", error=True
            )


def zip_paths(paths, exclude_paths: list[str] = []):
    common_prefix = find_common_prefix(paths)
    highest_dir, zip_file_name = get_zip_top_folder_and_name(common_prefix)
    temp_folder = create_temp_folder(highest_dir)
    try:
        ignore_paths = get_korbit_ignore(paths, exclude_paths)
        copy_files_to_temp_folder_with_ignore(paths, temp_folder, common_prefix, ignore_paths)
        add_repository_metadata(temp_folder)
        zip_file_name = create_zip_file(temp_folder, zip_file_name, ignore_paths)
    finally:
        rmtree_temporary_safely(temp_folder)
    return zip_file_name


def get_output_file(mode="a+") -> IO:
    return open(KORBIT_LOCAL_OUTPUT_LOG_FILE, mode)


def clean_output_file():
    if os.path.exists(KORBIT_LOCAL_OUTPUT_LOG_FILE):
        os.remove(KORBIT_LOCAL_OUTPUT_LOG_FILE)



import os
import platform
import site
import socket
import sys
import traceback

import click
import requests

from korbit import __version__
from korbit.constant import (
    KORBIT_INSTALLATION_SOURCE_BINARY,
    KORBIT_INSTALLATION_SOURCE_PIP,
    KORBIT_SCAN_TELEMETRY_URL,
)
from korbit.interface import INTERFACE_SOMETHING_WENT_WRONG_MSG
from korbit.login import authenticate_request, get_user_secret_id


def installation_source(command_path: str):
    bin_folder = os.path.dirname(sys.executable)
    binary_path = os.path.join(bin_folder, "korbit")
    site_packages_path = site.getsitepackages()[0]
    if os.path.exists(binary_path) and command_path == binary_path:
        if site_packages_path.startswith(os.path.commonprefix([bin_folder, site_packages_path])):
            return KORBIT_INSTALLATION_SOURCE_PIP
    return KORBIT_INSTALLATION_SOURCE_BINARY


def get_user_agent(command: list[str]):
    user_agent = {
        "os": platform.system(),
        "os_version": platform.release(),
        "architecture": platform.machine(),
        "python_version": sys.version,
        "hostname": socket.gethostname(),
        "cli_version": __version__,
        "command": " ".join(command) if command else "",
        "user_id": get_user_secret_id(),
    }
    if command:
        user_agent["installation_source"] = installation_source(command[0])
    else:
        user_agent["installation_source"] = "unknown"
    return user_agent


def send_telemetry(command: list[str], message: str, error: bool = False):
    log = {
        "version": __version__,
        "user_agent": {},
        "user_secret_id": get_user_secret_id(),
        "type": "ERROR" if error else "INFO",
        "message": message,
    }
    try:
        user_agent = get_user_agent(command)
        log["user_agent"] = user_agent
    except Exception:
        # if user agent isn't computable concat the traceback to the current message.
        log["message"] = f"{message}\n\nSomething wrong happened while computing user agent: {traceback.format_exc()}"

    try:
        authenticate_request(requests.post, KORBIT_SCAN_TELEMETRY_URL, json=log)
    except requests.exceptions.RequestException:
        # We don't want to raise the exception again because it will infinite loop
        click.echo(INTERFACE_SOMETHING_WENT_WRONG_MSG)



import getpass
import logging
import os
import sys
import traceback

import click
import requests

from korbit import __version__
from korbit.constant import (
    KORBIT_BACKEND_DEFAULT_LICENSE_LIMIT_REACHED_MSG,
    KORBIT_COMMAND_EXIT_CODE_AUTH_FAILED,
    KORBIT_COMMAND_EXIT_CODE_BRANCH_NOT_FOUND,
    KORBIT_COMMAND_EXIT_CODE_CHECK_FAILED,
    KORBIT_COMMAND_EXIT_CODE_DEPRECATED_CLI_VERSION,
    KORBIT_COMMAND_EXIT_CODE_GIT_IS_NOT_INSTALLED,
    KORBIT_COMMAND_EXIT_CODE_ISSUES_FOUND_WITHIN_THRESHOLD,
    KORBIT_COMMAND_EXIT_CODE_LICENSE_LIMIT_REACHED,
    KORBIT_COMMAND_EXIT_CODE_NO_GIT_REPOSITORY,
    KORBIT_COMMAND_EXIT_CODE_UNKNOWN_ERROR,
    KORBIT_COMMAND_EXIT_CODE_UNKNOWN_GIT_COMMAND_ERROR,
    KORBIT_LOCAL_FOLDER,
)
from korbit.interface import (
    INTERFACE_AUTH_COMMAND_HELP,
    INTERFACE_AUTH_INPUT_SECRET_ID,
    INTERFACE_AUTH_INPUT_SECRET_ID_HELP,
    INTERFACE_AUTH_INPUT_SECRET_KEY,
    INTERFACE_AUTH_INPUT_SECRET_KEY_HELP,
    INTERFACE_CHECK_FAILED_MSG,
    INTERFACE_SCAN_COMMAND_HELP,
    INTERFACE_SCAN_EXCLUDE_PATHS_HELP,
    INTERFACE_SCAN_EXIT_CODE_HELP,
    INTERFACE_SCAN_FINAL_REPORT_FOR_HEADLESS_MSG,
    INTERFACE_SCAN_FINAL_REPORT_PATH_MSG,
    INTERFACE_SCAN_HEADLESS_OUTPUT_REPORT_HELP,
    INTERFACE_SCAN_LICENSE_LIMIT_REACHED_MSG,
    INTERFACE_SCAN_NO_FILE_FOUND_MSG,
    INTERFACE_SCAN_PR_COMMAND_HELP,
    INTERFACE_SCAN_PR_EXIT_CODE_HELP,
    INTERFACE_SCAN_PR_GIT_BRANCH_NOT_FOUND_MSG,
    INTERFACE_SCAN_PR_GIT_COMMAND_ERROR_MSG,
    INTERFACE_SCAN_PR_GIT_IS_NOT_INSTALLED_MSG,
    INTERFACE_SCAN_PR_NOT_GIT_REPOSITORY_MSG,
    INTERFACE_SCAN_PREPARING_FOLDER_SCAN_MSG,
    INTERFACE_SCAN_REQUESTING_A_SCAN_MSG,
    INTERFACE_SCAN_THRESHOLD_CONFIDENCE_HELP,
    INTERFACE_SCAN_THRESHOLD_PRIORITY_HELP,
    INTERFACE_SCAN_UPLOAD_FAILED_DUE_TO_SIZE_MSG,
    INTERFACE_SOMETHING_WENT_WRONG_MSG,
)
from korbit.local_file import (
    ZipfileEmptyError,
    clean_output_file,
    get_output_file,
    repository_metadata_update,
    zip_paths,
)
from korbit.login import KorbitAuthError, store_credentials
from korbit.models.issue import IssueFilterThresholds
from korbit.models.report import Report
from korbit.scan import (
    display_report,
    display_scan_status,
    download_report,
    filter_issues_by_threshold,
    upload_file,
)
from korbit.telemetry import send_telemetry
from korbit.update import should_update

old_stdout, old_stderr = sys.stdout, sys.stderr
is_pr_scan = False


@click.version_option(__version__)
@click.group()
def cli():
    pass


def cleanup(headless: bool):
    repository_metadata_update({})
    sys.stdout, sys.stderr = old_stdout, old_stderr
    if headless:
        output_file = get_output_file("r+")
        click.echo(output_file.read())


@cli.command("login", help=INTERFACE_AUTH_COMMAND_HELP)
@click.option("-sid", "--secret-id", "--secret_id", default=None, help=INTERFACE_AUTH_INPUT_SECRET_ID_HELP)
@click.option("-skey", "--secret-key", "--secret_key", default=None, help=INTERFACE_AUTH_INPUT_SECRET_KEY_HELP)
@click.argument("client_secret_id", required=False, type=click.STRING)
@click.argument("client_secret_key", required=False, type=click.STRING)
@click.option("--verbose", is_flag=True, default=False, required=False)
def login(client_secret_id, client_secret_key, secret_id, secret_key, verbose):
    try:
        if should_update(sys.argv):
            return
        if not secret_id:
            if not client_secret_id:
                secret_id = input(INTERFACE_AUTH_INPUT_SECRET_ID)
            else:
                secret_id = client_secret_id
        if not secret_key:
            if not client_secret_key:
                secret_key = getpass.getpass(INTERFACE_AUTH_INPUT_SECRET_KEY)
            else:
                secret_key = client_secret_key
        store_credentials(secret_id, secret_key)
    except Exception:
        if verbose:
            logging.exception(INTERFACE_SOMETHING_WENT_WRONG_MSG)
        click.echo(INTERFACE_SOMETHING_WENT_WRONG_MSG)
        send_telemetry(
            sys.argv, f"{INTERFACE_SOMETHING_WENT_WRONG_MSG}\n{secret_id}\n{traceback.format_exc()}", error=True
        )


@cli.command("scan", help=INTERFACE_SCAN_COMMAND_HELP)
@click.option(
    "--threshold-priority", type=int, default=0, required=False, help=INTERFACE_SCAN_THRESHOLD_CONFIDENCE_HELP
)
@click.option(
    "--threshold-confidence", default=0, type=int, required=False, help=INTERFACE_SCAN_THRESHOLD_PRIORITY_HELP
)
@click.option("--exit-code", is_flag=True, default=None, required=False, help=INTERFACE_SCAN_EXIT_CODE_HELP)
@click.option(
    "--headless",
    is_flag=True,
    default=None,
    required=False,
    help=INTERFACE_SCAN_HEADLESS_OUTPUT_REPORT_HELP,
)
@click.option(
    "--exclude-paths",
    type=str,
    help=INTERFACE_SCAN_EXCLUDE_PATHS_HELP,
    required=False,
    default=".git/*,**/.git/*,**/.git,node_modules/*,**/node_modules/*,**/node_modules",
)
@click.option(
    "--verbose",
    is_flag=True,
    required=False,
    default=False,
)
@click.argument("paths", nargs=-1, type=click.Path(exists=True))
def scan(paths, threshold_priority, threshold_confidence, exit_code, headless, exclude_paths, verbose):
    global old_stdout, old_stderr, is_pr_scan
    os.makedirs(KORBIT_LOCAL_FOLDER, exist_ok=True)
    try:
        if should_update(sys.argv):
            if exit_code:
                sys.exit(KORBIT_COMMAND_EXIT_CODE_DEPRECATED_CLI_VERSION)
            return
        clean_output_file()
        if headless:
            output_file = get_output_file()
            sys.stdout = output_file
            sys.stderr = output_file

        click.echo(INTERFACE_SCAN_PREPARING_FOLDER_SCAN_MSG.format(paths=" ".join(paths)))
        zip_file_path = zip_paths(list(paths), exclude_paths=exclude_paths.split(","))
        click.echo(zip_file_path)
        click.echo(INTERFACE_SCAN_REQUESTING_A_SCAN_MSG.format(path=zip_file_path))
        scan_id = upload_file(zip_file_path, is_pr_scan=is_pr_scan)

        if not scan_id:
            click.echo(INTERFACE_CHECK_FAILED_MSG)
            if exit_code:
                sys.exit(KORBIT_COMMAND_EXIT_CODE_CHECK_FAILED)
            cleanup(headless)
            return

        display_scan_status(scan_id, headless)
        issues = download_report(scan_id)
        report = Report.from_json(issues)

        issue_thresholds = IssueFilterThresholds(priority=threshold_priority, confidence=threshold_confidence)
        report = filter_issues_by_threshold(report, issue_thresholds)
        display_report(report, headless=headless)

        if not report.is_successful() and exit_code:
            click.echo(INTERFACE_SCAN_FINAL_REPORT_FOR_HEADLESS_MSG.format(path=report.report_path))
            sys.exit(KORBIT_COMMAND_EXIT_CODE_ISSUES_FOUND_WITHIN_THRESHOLD)

        click.echo(INTERFACE_SCAN_FINAL_REPORT_PATH_MSG.format(path=report.report_path))
    except KorbitAuthError as e:
        if verbose:
            logging.exception(str(e))
        click.echo(str(e))
        send_telemetry(
            sys.argv,
            f"{e}\n{traceback.format_exc()}",
            error=False,
        )
        if exit_code:
            sys.exit(KORBIT_COMMAND_EXIT_CODE_AUTH_FAILED)
    except requests.exceptions.HTTPError as e:
        if e.response.status_code in [413, 400]:
            click.echo(INTERFACE_SCAN_UPLOAD_FAILED_DUE_TO_SIZE_MSG)
            if exit_code:
                sys.exit(KORBIT_COMMAND_EXIT_CODE_LICENSE_LIMIT_REACHED)
        elif e.response.status_code == 402:
            message = e.response.json().get("message")
            if not message or message == KORBIT_BACKEND_DEFAULT_LICENSE_LIMIT_REACHED_MSG:
                message = INTERFACE_SCAN_LICENSE_LIMIT_REACHED_MSG
            click.echo(message)
            send_telemetry(sys.argv, f"{message}\n{e}\n{traceback.format_exc()}", error=False)
            if exit_code:
                sys.exit(KORBIT_COMMAND_EXIT_CODE_LICENSE_LIMIT_REACHED)
        else:
            if verbose:
                logging.exception(INTERFACE_SOMETHING_WENT_WRONG_MSG)
            click.echo(INTERFACE_SOMETHING_WENT_WRONG_MSG)
            send_telemetry(sys.argv, f"{INTERFACE_SOMETHING_WENT_WRONG_MSG}\n{e}\n{traceback.format_exc()}", error=True)
            if exit_code:
                sys.exit(KORBIT_COMMAND_EXIT_CODE_UNKNOWN_ERROR)
    except ZipfileEmptyError:
        click.echo(INTERFACE_SCAN_NO_FILE_FOUND_MSG)
    except Exception as e:
        if verbose:
            logging.exception(INTERFACE_SOMETHING_WENT_WRONG_MSG)
        click.echo(INTERFACE_SOMETHING_WENT_WRONG_MSG)
        send_telemetry(sys.argv, f"{INTERFACE_SOMETHING_WENT_WRONG_MSG}\n{e}\n{traceback.format_exc()}", error=True)
        if exit_code:
            sys.exit(KORBIT_COMMAND_EXIT_CODE_UNKNOWN_ERROR)
    finally:
        cleanup(headless)


@cli.command("scan-pr", help=INTERFACE_SCAN_PR_COMMAND_HELP)
@click.option(
    "--threshold-priority", type=int, default=0, required=False, help=INTERFACE_SCAN_THRESHOLD_CONFIDENCE_HELP
)
@click.option(
    "--threshold-confidence", default=0, type=int, required=False, help=INTERFACE_SCAN_THRESHOLD_PRIORITY_HELP
)
@click.option("--exit-code", is_flag=True, default=None, required=False, help=INTERFACE_SCAN_PR_EXIT_CODE_HELP)
@click.option(
    "--headless",
    is_flag=True,
    default=None,
    required=False,
    help=INTERFACE_SCAN_HEADLESS_OUTPUT_REPORT_HELP,
)
@click.option(
    "--exclude-paths",
    type=str,
    help=INTERFACE_SCAN_EXCLUDE_PATHS_HELP,
    required=False,
    default="**/.git/*,**/.git",
)
@click.option(
    "--verbose",
    is_flag=True,
    required=False,
    default=False,
)
@click.argument(
    "repository-path",
    type=click.Path(exists=True),
    required=False,
    default=".",
)
@click.argument("compare-branch", type=str, required=False, default="origin/master")
def scan_pr(
    repository_path,
    compare_branch,
    threshold_priority,
    threshold_confidence,
    exit_code,
    headless,
    exclude_paths,
    verbose,
):
    global is_pr_scan
    os.makedirs(KORBIT_LOCAL_FOLDER, exist_ok=True)
    try:
        import git

        from korbit.version_control import (
            BranchNotFound,
            get_diff_paths,
            get_repository_metadata_with_diff,
        )
    except ImportError:
        click.echo(INTERFACE_SCAN_PR_GIT_IS_NOT_INSTALLED_MSG)
        if exit_code:
            sys.exit(KORBIT_COMMAND_EXIT_CODE_GIT_IS_NOT_INSTALLED)
        return
    diff_paths = []
    try:
        if should_update(sys.argv):
            if exit_code:
                sys.exit(KORBIT_COMMAND_EXIT_CODE_DEPRECATED_CLI_VERSION)
            return
        diff_paths = get_diff_paths(repository_path, compare_branch=compare_branch)
        repository_metadata_update(get_repository_metadata_with_diff(repository_path, compare_branch))
    except BranchNotFound as e:
        if verbose:
            logging.exception(INTERFACE_SCAN_PR_GIT_BRANCH_NOT_FOUND_MSG.format(branch=e.branch_name))
        click.echo(INTERFACE_SCAN_PR_GIT_BRANCH_NOT_FOUND_MSG.format(branch=e.branch_name))
        send_telemetry(
            sys.argv,
            f"{INTERFACE_SCAN_PR_GIT_BRANCH_NOT_FOUND_MSG.format(branch=e.branch_name)}\n{e}\n{traceback.format_exc()}",
            error=False,
        )
        if exit_code:
            sys.exit(KORBIT_COMMAND_EXIT_CODE_BRANCH_NOT_FOUND)
    except git.InvalidGitRepositoryError:
        if verbose:
            logging.exception(INTERFACE_SCAN_PR_NOT_GIT_REPOSITORY_MSG)
        click.echo(INTERFACE_SCAN_PR_NOT_GIT_REPOSITORY_MSG)
        send_telemetry(sys.argv, f"{INTERFACE_SCAN_PR_NOT_GIT_REPOSITORY_MSG}\n{traceback.format_exc()}", error=True)
        if exit_code:
            sys.exit(KORBIT_COMMAND_EXIT_CODE_NO_GIT_REPOSITORY)
    except git.GitCommandError as e:
        error_message = e.stderr.strip()
        if verbose:
            logging.exception(
                INTERFACE_SCAN_PR_GIT_COMMAND_ERROR_MSG.format(error_message=error_message, status_code=str(e.status))
            )
        click.echo(
            INTERFACE_SCAN_PR_GIT_COMMAND_ERROR_MSG.format(error_message=error_message, status_code=str(e.status))
        )
        send_telemetry(
            sys.argv, f"{INTERFACE_SCAN_PR_GIT_COMMAND_ERROR_MSG}\n{e}\n{traceback.format_exc()}", error=True
        )
        if exit_code:
            sys.exit(KORBIT_COMMAND_EXIT_CODE_UNKNOWN_GIT_COMMAND_ERROR)
    except Exception as e:
        if verbose:
            logging.exception(INTERFACE_SOMETHING_WENT_WRONG_MSG)
        send_telemetry(sys.argv, f"{INTERFACE_SOMETHING_WENT_WRONG_MSG}\n{e}\n{traceback.format_exc()}", error=True)
        click.echo(INTERFACE_SOMETHING_WENT_WRONG_MSG)
        if exit_code:
            sys.exit(KORBIT_COMMAND_EXIT_CODE_UNKNOWN_ERROR)

    if not diff_paths:
        click.echo(INTERFACE_SCAN_NO_FILE_FOUND_MSG)
        return

    is_pr_scan = True
    scan_command_args = diff_paths
    scan_command_args += [
        f"--threshold-priority={threshold_priority}",
        f"--threshold-confidence={threshold_confidence}",
    ]
    if exit_code:
        scan_command_args.append("--exit-code")
    if headless:
        scan_command_args.append("--headless")
    if verbose:
        scan_command_args.append("--verbose")
    scan_command_args.append(f"--exclude-paths={exclude_paths}")
    scan(scan_command_args)


if __name__ == "__main__":
    cli()



import os

# API Endpoints
KORBIT_HOST = os.getenv("KORBIT_HOST", "https://mentor.korbit.ai:8000")
KORBIT_SCAN_REPORT_URL = f"{KORBIT_HOST}/database/v3/scans"
KORBIT_SCAN_VERSION_URL = f"{KORBIT_HOST}/cli"
KORBIT_SCAN_TELEMETRY_URL = f"{KORBIT_HOST}/cli/telemetry"
KORBIT_CODE_ANALYSIS_SERVICE = f"{KORBIT_HOST}/code-analysis/check"
KORBIT_CODE_ANALYSIS_CHECK = f"{KORBIT_CODE_ANALYSIS_SERVICE}/files"
KORBIT_CODE_ANALYSIS_PULL_REQUEST_CHECK = f"{KORBIT_CODE_ANALYSIS_SERVICE}/pull_request/files"

# Local paths
KORBIT_LOCAL_FOLDER = ".korbit"
KORBIT_LOCAL_OUTPUT_LOG_FILE = f"{KORBIT_LOCAL_FOLDER}/scan.log"
KORBIT_LOCAL_REPOSITORY_METADATA_FILENAME = "repository_metadata.json"
KORBIT_LOCAL_REPOSITORY_METADATA_PATH = f"{KORBIT_LOCAL_FOLDER}/{KORBIT_LOCAL_REPOSITORY_METADATA_FILENAME}"

# Exit codes
KORBIT_COMMAND_EXIT_CODE_UNKNOWN_ERROR = 90
KORBIT_COMMAND_EXIT_CODE_ISSUES_FOUND_WITHIN_THRESHOLD = 91
KORBIT_COMMAND_EXIT_CODE_CHECK_FAILED = 92
KORBIT_COMMAND_EXIT_CODE_AUTH_FAILED = 93
KORBIT_COMMAND_EXIT_CODE_FILE_TOO_LARGE = 94
KORBIT_COMMAND_EXIT_CODE_LICENSE_LIMIT_REACHED = 95
KORBIT_COMMAND_EXIT_CODE_DEPRECATED_CLI_VERSION = 96

KORBIT_COMMAND_EXIT_CODE_UNKNOWN_GIT_COMMAND_ERROR = 190
KORBIT_COMMAND_EXIT_CODE_NO_GIT_REPOSITORY = 191
KORBIT_COMMAND_EXIT_CODE_BRANCH_NOT_FOUND = 192
KORBIT_COMMAND_EXIT_CODE_GIT_IS_NOT_INSTALLED = 193

# Backend
KORBIT_BACKEND_DEFAULT_LICENSE_LIMIT_REACHED_MSG = "Invalid or missing license."
KORBIT_INSTALLATION_SOURCE_BINARY = "binary"
KORBIT_INSTALLATION_SOURCE_PIP = "pip"



from setuptools import find_packages, setup

setup(
    name="korbit-mentor",
    version="3.6.7",
    packages=find_packages(),
    long_description=open("PUBLIC_README.md").read(),
    long_description_content_type="text/markdown",
    description="Korbit mentor CLI tool will allow you to analyze any local files.",
    author="Korbit Technologies Inc.",
    author_email="team@korbit.ai",
    url="https://www.korbit.ai",
    keywords=["SOFTWARE", "DEVELOPMENT", "MENTOR", "ENGINEER"],
    install_requires=["validators", "beautifulsoup4", "click", "rich", "requests", "gitpython"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "Programming Language :: Python :: 3.11",
    ],
    entry_points={
        "console_scripts": [
            "korbit = korbit.cli:cli",  # Define the entry point for your CLI
        ],
    },
)



import traceback

import requests

from korbit import __version__
from korbit.constant import KORBIT_INSTALLATION_SOURCE_PIP, KORBIT_SCAN_VERSION_URL
from korbit.interface import INTERFACE_UPDATE_REQUIRED_MSG, print_markdown_message
from korbit.login import CLIVersionDeprecated, authenticate_request
from korbit.telemetry import get_user_agent, send_telemetry


def should_update(command: list[str]):
    try:
        user_agent = get_user_agent(command)
    except Exception:
        send_telemetry([""], f"User agent not computable: {traceback.format_exc()}", error=True)
        user_agent = {}
    try:
        query_params = {
            "version": __version__,
            "architecture": user_agent.get("architecture", "x86_64"),
            "os": user_agent.get("os", "Darwin"),
        }
        headers = {"User-Agent": str(user_agent)}
        authenticate_request(
            requests.get, f"{KORBIT_SCAN_VERSION_URL}?version={__version__}", params=query_params, headers=headers
        )
        return False
    except CLIVersionDeprecated as e:
        if user_agent.get("installation_source") == KORBIT_INSTALLATION_SOURCE_PIP:
            print_markdown_message(INTERFACE_UPDATE_REQUIRED_MSG)
        else:
            print_markdown_message(str(e))
        return True



import base64
import json
import os
from pathlib import Path
from typing import Callable, Optional

import requests

from korbit.interface import (
    INTERFACE_AUTH_MISSING_CREDENTIALS_MSG,
    INTERFACE_AUTH_UNAUTHORIZED_CREDENTIALS_MSG,
    INTERFACE_UPDATE_REQUIRED_MSG,
)

KORBIT_CREDENTIAL_FILE = os.path.expanduser("~/.korbit/credentials")


class KorbitAuthError(Exception):
    pass


class CLIVersionDeprecated(Exception):
    pass


def store_credentials(secret_id, secret_key):
    """
    Store user credentials for future usage of scan command.
    """
    os.makedirs(Path(KORBIT_CREDENTIAL_FILE).parent, exist_ok=True)
    with open(KORBIT_CREDENTIAL_FILE, "w+") as credential_file:
        json.dump({"secret_id": secret_id, "secret_key": secret_key}, credential_file)


def get_credential() -> dict:
    credentials = {}
    if os.path.exists(KORBIT_CREDENTIAL_FILE):
        with open(KORBIT_CREDENTIAL_FILE, "r+") as credential_file:
            credentials = json.loads(credential_file.read())
    return credentials


def compute_user_token():
    credentials = get_credential()
    secret_id = os.getenv("KORBIT_SECRET_ID", credentials.get("secret_id"))
    secret_key = os.getenv("KORBIT_SECRET_KEY", credentials.get("secret_key"))

    if not secret_id or not secret_key:
        raise KorbitAuthError(INTERFACE_AUTH_MISSING_CREDENTIALS_MSG)
    return base64.b64encode(f"{secret_id}:{secret_key}".encode()).decode()


def get_user_secret_id() -> Optional[str]:
    credentials = get_credential()
    return os.getenv("KORBIT_SECRET_ID", credentials.get("secret_id"))


def authenticate_request(method: Callable[[str], requests.Response], url: str, **kwargs) -> requests.Response:
    headers = kwargs.pop("headers", {})
    if not headers.get("Authorization"):
        try:
            headers["Authorization"] = f"Basic {compute_user_token()}"
        except KorbitAuthError:
            pass

    kwargs["headers"] = headers
    response = method(url, **kwargs)
    if response.status_code in [401, 403]:
        raise KorbitAuthError(INTERFACE_AUTH_UNAUTHORIZED_CREDENTIALS_MSG)
    if response.status_code == 410:
        raise CLIVersionDeprecated(response.json().get("message", INTERFACE_UPDATE_REQUIRED_MSG))
    response.raise_for_status()
    return response
